## source('~/Dropbox/R_Pres_Demo_02.R')
## source('~/Dropbox/R_Pres_Demo_02.R', echo=T)
## Based on ~/Dropbox/R_Pres_Demo_01.R

## Creating a function
## ----------------------------------------
## Pause (sleep) for specified number of seconds
testit <- function(x)
{
    p1 <- proc.time()
    Sys.sleep(x)
    proc.time() - p1 # The cpu usage should be negligible
}

testit(1)   # Pause (sleep) for specified number of seconds
## --------------------------------------------------

class(1)
class(TRUE)
class(rnorm(100))
class(NA)
class("foo")

x<- log(2.843432) *pi 
x

sqrt(x) 

floor(x)        # largest integer less than or equal to x 
ceiling(x)      # smallest integer greater than or equal to x

testit(10)   # Pause (sleep) for specified number of seconds

# R can also handle complex numbers.

x<-3+2i 
Re(x)           # Real part of the complex number x

Im(x)           # Imaginary part of x (use capital I) 

y<- -1+1i 
x+y 

x*y 

testit(10)   # Pause (sleep) for specified number of seconds

# cars is one of the built-in data sets 

 lin.mod <- lm(dist~speed,data=cars) # lm is linear model for linear regression
 lin.mod

# Call:
# lm(formula = dist ~ speed, data = cars)

# Coefficients:
# (Intercept)        speed  
#     -17.579        3.932  

plot(cars$speed, cars$dist)
testit(10)   # Pause (sleep) for specified number of seconds
abline(lin.mod)

# To learn more: 
#     ?lm
#     ?plot
#     ?abline

# Data classes go beyond the atomic classes

set.seed(2014)
x <- rnorm(100)
y <- x + rnorm(100)
fit <- lm(y ~ x)     # linear regression model 
fit

# Call:
# lm(formula = y ~ x)

# Coefficients:
# (Intercept)            x  
#     0.01793      1.09070  

testit(10)   # Pause (sleep) for specified number of seconds

class(fit)
# [1] "lm"

plot(x,y)

testit(10)   # Pause (sleep) for specified number of seconds

# Other examples of built-in functions 

x <- seq(-2, 2, 0.05)

y1 <- pnorm(x)

y2 <- pnorm(x,1,1)

plot(x,y1,type="l",col="red", main="pnorm(x) in red vs x <- [seq(-2, 2, 0.05)]")

testit(15)   # Pause (sleep) for specified number of seconds

lines(x,y2,col="green")

# Modified code to use two different functions

 x  <- seq(-2, 2, 0.05)

 y1 <- tanh(x)

 y2 <- sin(x)

 plot(x,y1,type="l",col="red", main="tanh(x) in red & sin(x) in green vs x <- seq(-2, 2, 0.05)")

testit(7)   # Pause (sleep) for specified number of seconds

 lines(x,y2,col="green")

testit(7)   # Pause (sleep) for specified number of seconds

# Modified code to use two different functions

 x  <- seq(-2*pi, 2*pi, 0.05)

 y1 <- tanh(x)

 y2 <- sin(x)

 plot(x,y1,type="l",col="red", main="tanh(x) in red & sin(x) in green vs x <- seq(-2*pi, 2*pi, 0.05)")

testit(7)   # Pause (sleep) for specified number of seconds

 lines(x,y2,col="green")

testit(20)   # Pause (sleep) for specified number of seconds

# Example of a build your own function

fib <- function(n=20){
  if(n<3){
  return(c(1,1))
  } else{
    fib <- rep(0, n)
    for(i in 1:n){
      if(i <= 2){
        fib[i] <- 1
      } else{
        fib[i] <- fib[i-1] + fib[i-2]
      }
    }
  }
  return(fib)
}

fib()
# [1]    1    1    2    3    5    8   13   21   34   55   89  144  233  377  610  987 1597 2584 4181 6765

fib(5)
# [1] 1 1 2 3 5

testit(15)   # Pause (sleep) for specified number of seconds

# Packages - An example (demo)

# install.packages('quantmod')  # to access financial data
require(quantmod)

# Get S&P closing prices and daily volume since 1/1/1950 
sp500 <- getSymbols("^GSPC", auto.assign = FALSE, from="1950-01-01")[, (4:5)]

colnames(sp500) <- c('SP500', 'SPvolume')

tail(sp500, 10)    # to see most recent data (last 10 days)

head(sp500)    # to see oldest data (first 6 days)

#-------
testit(5)   # Pause (sleep) for specified number of seconds

# R Learning Module - Subsetting Data
# See http://www.ats.ucla.edu/stat/r/modules/subsetting.htm

hsb2.small <- read.csv("http://www.ats.ucla.edu/stat/data/hsb2_small.csv")

# using the names function to see names of the variables 
# and which column of data to which they correspond

names(hsb2.small)
##  [1] "id"      "female"  "race"    "ses"     "schtyp"  "prog"    "read"   
##  [8] "write"   "math"    "science" "socst"

(hsb3 <- hsb2.small[, c(1, 7, 8)])    # Print hsb3

(hsb4 <- hsb2.small[, 1:4])

(hsb5 <- hsb2.small[1:10, ])

(hsb6 <- hsb2.small[hsb2.small$ses == 1, ])

(hsb7 <- hsb2.small[hsb2.small$id %in% c(12, 48, 86, 11, 20, 195), ])

(hsb8 <- hsb2.small[with(hsb2.small, ses == 3 & female == 0), ])

#-------
testit(5)   # Pause (sleep) for specified number of seconds

# Simple plot example:

# Get stats package.  See ?stats for more info.
require(stats)

# Plot data from the cars data set
plot(cars)

testit(5)   # Pause (sleep) for specified number of seconds 

# Add lines using the lowess function. 
# See ?lowess and ?lines
lines(lowess(cars))

#-------

# My quantmod example:

# Don't forget to install the necessary packages
# for example, install.packages('quantmod')

# Required package is quantmod
require(quantmod)

# Get S&P data since 1/1/1950
sp500 <- getSymbols("^GSPC", auto.assign = FALSE, from="1950-01-01")

tail(sp500)
#           GSPC.Open GSPC.High GSPC.Low GSPC.Close GSPC.Volume GSPC.Adjusted
#2013-11-20   1789.59   1795.73  1777.23    1781.37  3109140000       1781.37
#2013-11-21   1783.52   1797.16  1783.52    1795.85  3256630000       1795.85
#2013-11-22   1797.21   1804.84  1794.70    1804.76  3055140000       1804.76
#2013-11-25   1806.33   1808.10  1800.58    1802.48  2998540000       1802.48
#2013-11-26   1802.87   1808.42  1800.77    1802.75  3427120000       1802.75
#2013-11-27   1803.48   1808.27  1802.77    1807.23  2613590000       1807.23

testit(5)   # Pause (sleep) for specified number of seconds

# Not interested in all columns, just two 
# Get S&P closing prices and corresponding daily volume since 1/1/1950 
sp500 <- getSymbols("^GSPC", auto.assign = FALSE, from="1950-01-01")[, (4:5)]

# Change column names to something more understandable
colnames(sp500) <- c("SP500", "SPvolume")

tail(sp500)
#             SP500   SPvolume
#2013-11-20 1781.37 3109140000
#2013-11-21 1795.85 3256630000
#2013-11-22 1804.76 3055140000
#2013-11-25 1802.48 2998540000
#2013-11-26 1802.75 3427120000
#2013-11-27 1807.23 2613590000

# plot SP500
plot(sp500$SP500)

testit(5)   # Pause (sleep) for specified number of seconds

# plot SP500 using a log scale
plot(sp500$SP500, log="y")

# ---------------
testit(5)   # Pause (sleep) for specified number of seconds

plot(tail(sp500$SP500, 256, log="y"), main="SP500 Index (last 12 months)")

# ---------------
testit(10)   # Pause (sleep) for specified number of seconds

plot(tail(sp500$SP500, 128, log="y"), main="SP500 Index (last 6 months)")

# ---------------
testit(10)   # Pause (sleep) for specified number of seconds

plot(tail(sp500$SP500, 63, log="y"), main="SP500 Index (last 3 months)")

summary(sp500)

tail(sp500, 15)

# ---------------
# Calculate the S&P percentage below the all time high
SPdnPCT <-  100 * (1-(tail(sp500$SP500, 1)) / max(sp500$SP500))
#print(round(SPdnPCT, digits=2))


string1 <- "The S&P 500 Index is now down "

string2 <- "percent from the all time high." 

#print(string1); print(round(SPdnPCT, digits=2)); print(string2)

## paste examples:
#paste("A", 1:6, sep = "")
#     stopifnot(identical(paste ("A", 1:6, sep = ""),
#                         paste0("A", 1:6)))
#     paste("Today is", date())

## A function to clear the console in R
clc <- function() cat(rep("\n",50))
clc()

print(paste(string1, round(SPdnPCT, digits=2), string2)) 

# ---------------

